#include "draw.h"


void drawCursor(void *buffer)
{
  int i;
  int color=0xF;
  if(has_colors)
    color=0xFFFF;
  
  for(i=0;i<=19;i++)
  {
    bufSetPixel(buffer,i+149,118,color);
    bufSetPixel(buffer,i+149,119,color);
    bufSetPixel(buffer,158,i+109,color);
    bufSetPixel(buffer,159,i+109,color);
  }
}

void drawCubeIndicator(void *buffer,int color)
{
  drawTriangle(buffer,color,309,0,319,0,319,9);
}

void drawCube(void *buffer,double *vertices,int *pos,int *faces,int *colors)
{
  if(isCubeOnScreen(pos))
  {
    int i=0,j=0;
    int a=0,b=0,c=0,d=0;
    double e=0.0,f=0.0,g=0.0;
    double dist[6]={0.0};
    
    int facesImg[6*4]={0};
    int colorsImg[6]={0};
    
    memcpy(facesImg,faces,24*sizeof(int));
    memcpy(colorsImg,colors,6*sizeof(int));
    
    for(i=0;i<6;i++)
    {
      e=vertices[faces[i*4]*3-3]/2.0+vertices[faces[i*4+2]*3-3]/2.0;
      f=vertices[faces[i*4]*3-2]/2.0+vertices[faces[i*4+2]*3-2]/2.0;
      g=vertices[faces[i*4]*3-1]/2.0+vertices[faces[i*4+2]*3-1]/2.0;
      dist[i]=e*e+f*f+g*g;
    }
    
    for(i=0;i<5;i++)
    {
      for(j=0;j<5-i;j++)
      {
        if(dist[j]>dist[j+1])
        {
          e=dist[j];
          dist[j]=dist[j+1];
          dist[j+1]=e;
          invertTableElems(facesImg,4*j,4*j+4);
          invertTableElems(facesImg,4*j+1,4*j+5);
          invertTableElems(facesImg,4*j+2,4*j+6);
          invertTableElems(facesImg,4*j+3,4*j+7);
          invertTableElems(colorsImg,j,j+1);
        }
      }
    }
    
    i=2;
    while(i>=0)
    {
      a=facesImg[i*4]-1;
      b=facesImg[i*4+1]-1;
      c=facesImg[i*4+2]-1;
      d=facesImg[i*4+3]-1;
      drawTriangle(buffer,colorsImg[i],pos[a*2],pos[a*2+1],pos[b*2],pos[b*2+1],pos[c*2],pos[c*2+1]);
      drawTriangle(buffer,colorsImg[i],pos[d*2],pos[d*2+1],pos[a*2],pos[a*2+1],pos[c*2],pos[c*2+1]);
      i--;
    }
  }
}



int isCubeOnScreen(int *pos)
{
  int test=1;
  if(pos[0]<0 && pos[2]<0 && pos[4]<0 && pos[6]<0 && pos[8]<0 && pos[10]<0 && pos[12]<0 && pos[14]<0)
  {
    test=0;
  }
  else if(pos[1]<0 && pos[3]<0 && pos[5]<0 && pos[7]<0 && pos[9]<0 && pos[11]<0 && pos[13]<0 && pos[15]<0)
  {
    test=0;
  }
  else if(pos[0]>319 && pos[2]>319 && pos[4]>319 && pos[6]>319 && pos[8]>319 && pos[10]>319 && pos[12]>319 && pos[14]>319)
  {
    test=0;
  }
  else if(pos[1]>239 && pos[3]>239 && pos[5]>239 && pos[7]>239 && pos[9]>239 && pos[11]>239 && pos[13]>239 && pos[15]>239)
  {
    test=0;
  }
  return test;
}



void drawTriangle(void *buffer,int color,int x1,int y1,int x2,int y2,int x3,int y3)
{
  int i=0;
  int start=1;
  if(y1>y2)
  {
    invertVars(&y1,&y2);
    invertVars(&x1,&x2);
  }
  if(y1>y3)
  {
    invertVars(&y1,&y3);
    invertVars(&x1,&x3);
  }
  if(y2>y3)
  {
    invertVars(&y2,&y3);
    invertVars(&x2,&x3);
  }
  if(y2>0)
  {
    start=0;
    if(y1<0)
      start=-y1;
    for(i=start;i<=y2-y1;i++)
    {
      bufHorizLine(buffer,x1+(x2-x1)*i/(y2-y1),y1+i,x1+(x3-x1)*i/(y3-y1),color);
    }
  }
  if(y3>0)
  {
    start=0;
    if(y2<0)
      start=-y2;
    for(i=start;i<=y3-y2;i++)
    {
      bufHorizLine(buffer,x2+(x3-x2)*i/(y3-y2),y2+i,x1+(x3-x1)*(y2-y1+i)/(y3-y1),color);
    }
  }
}

