#include "drawChunk.h"



void heap(double *dist,int *index,int i,int n)
{
  int k=i;
  int j=2*k;
  while(j<=n)
  {
    if(j<n && dist[j]>dist[j+1])
    {
      j++;
    }
    if(dist[k]>dist[j])
    {
      invertTableDoubles(dist,k,j);
      invertTableElems(index,k,j);
      k=j;
      j=2*k;
    }
    else
    {
      break;
    }
  }
}




void sortCubePos(double *cubePos,uint8_t *cubeList,int size)
{
  int i=0;
  double *dist=malloc(size*sizeof(double));
  int *index=malloc(size*sizeof(int));
  
  //calculates distance of each cube
  for(i=0;i<size;i++)
  {
    dist[i]=cubePos[i*3]*cubePos[i*3]+cubePos[i*3+1]*cubePos[i*3+1]+cubePos[i*3+2]*cubePos[i*3+2];
    index[i]=i;
  }
  
  //heap sort
  for(i=size/2-1;i>=0;i--)
  {
    heap(dist,index,i,size-1);
  }
  for(i=size-1;i>=1;i--)
  {
    invertTableDoubles(dist,i,0);
    invertTableElems(index,i,0);
    heap(dist,index,0,i-1);
  }
  
  //sort cubes with given indexes
  double *cubePosImg=malloc(size*3*sizeof(double));
  memcpy(cubePosImg,cubePos,size*3*sizeof(double));
  
  uint8_t *cubeListImg=malloc(size*sizeof(uint8_t));
  memcpy(cubeListImg,cubeList,size*sizeof(uint8_t));
  
  for(i=0;i<size;i++)
  {
    cubePos[i*3]=cubePosImg[index[i]*3];
    cubePos[i*3+1]=cubePosImg[index[i]*3+1];
    cubePos[i*3+2]=cubePosImg[index[i]*3+2];
    cubeList[i]=cubeListImg[index[i]];
  }
  
  free(dist);
  free(index);
  free(cubePosImg);
  free(cubeListImg);
}



int isCubeVisible(uint8_t *chunk,int cubeX,int cubeY,int cubeZ,int index)
{
  int visible=0;
  if(cubeZ>=CUBE_CHUNK_Z/2)
    visible+=(chunk[index-CUBE_CHUNK_X*CUBE_CHUNK_Y]==0) ? 1 : 0;
  if(cubeZ<=CUBE_CHUNK_Z/2)
    visible+=(chunk[index+CUBE_CHUNK_X*CUBE_CHUNK_Y]==0) ? 1 : 0;
  if(cubeX>=CUBE_CHUNK_X/2)
    visible+=(chunk[index-CUBE_CHUNK_Y]==0) ? 1 : 0;
  if(cubeX<=CUBE_CHUNK_X/2)
    visible+=(chunk[index+CUBE_CHUNK_Y]==0) ? 1 : 0;
  if(cubeY>=CUBE_CHUNK_Y/2)
    visible+=(chunk[index-1]==0) ? 1 : 0;
  if(cubeY<=CUBE_CHUNK_Y/2)
    visible+=(chunk[index+1]==0) ? 1 : 0;
  return visible;
}



void computeCubeWorldChunk(uint8_t *chunk,double *cubePos,uint8_t *cubeList,int *size,double xRotation,double zRotation,pos playerPos)
{
  int i=0,j=0,k=0; //iterators
  int index=0,n=0,displayed=0; //utils
  
  playerPos.x-=floor(playerPos.x/2.0)*2.0-(double)CUBE_CHUNK_X;
  playerPos.y-=floor(playerPos.y/2.0)*2.0-(double)CUBE_CHUNK_Y;
  playerPos.z-=floor(playerPos.z/2.0)*2.0-(double)CUBE_CHUNK_Z-2.0;
  
  //loop to see all cubes
  for(i=0;i<CUBE_CHUNK_Z;i++)
  {
    for(j=0;j<CUBE_CHUNK_X;j++)
    {
      for(k=0;k<CUBE_CHUNK_Y;k++)
      {
        //optimization of cubes display
        index=i*CUBE_CHUNK_Y*CUBE_CHUNK_X+j*CUBE_CHUNK_Y+k;
        displayed=isCubeVisible(chunk,j,k,i,index);
        
        if(chunk[index]!=0 && displayed)
        {
          cubePos[n*3]=(double)j*2.0+1.0;
          cubePos[n*3+1]=(double)k*2.0+1.0;
          cubePos[n*3+2]=(double)i*2.0+1.0;
          cubeList[n]=chunk[index];
          n=n+1;
        }
      }
    }
  }
  translate(cubePos,n,-playerPos.x,-playerPos.y,-playerPos.z);
  rotateZ(cubePos,n,zRotation);
  rotateX(cubePos,n,xRotation);
  sortCubePos(cubePos,cubeList,n);
  *size=n;
}





void drawCubeList(void *buffer,double *cubePos,uint8_t *cubeList,int size,double xRotation,double zRotation)
{
  //one cube -- to translate to each pos of cubePos
  double cubeVertices[8*3]={1.0,1.0,1.0,  1.0,1.0,-1.0,  1.0,-1.0,1.0,  1.0,-1.0,-1.0,  -1.0,1.0,1.0,  -1.0,1.0,-1.0,  -1.0,-1.0,1.0,  -1.0,-1.0,-1.0};
  double cubeVerticesImg[8*3]={0.0};
  
  int cubeFaces[6*4]={1,2,4,3,  1,5,6,2,  1,3,7,5,  8,7,3,4,  8,7,5,6,  8,4,2,6};
  int colors[6]={2,4,6,8,10,12};
  int verticesPos[8*2]={0};
  int i=0;
  
  rotateZ(cubeVertices,8,zRotation);
  rotateX(cubeVertices,8,xRotation);
  
  //draw all cubes
  for(i=0;i<size;i++)
  {
    memcpy(cubeVerticesImg,cubeVertices,24*sizeof(double));
    
    translate(cubeVerticesImg,8,cubePos[i*3],cubePos[i*3+1],cubePos[i*3+2]);
    
    colors[0]=(int)(getCubeColor(cubeList[i]));
    colors[1]=shadeColor(colors[0],1);
    colors[2]=shadeColor(colors[0],-1);
    colors[3]=shadeColor(colors[0],1);
    colors[4]=colors[0];
    colors[5]=shadeColor(colors[0],-1);
    
    computeVertices(cubeVerticesImg,verticesPos,8);
    if(verticesPos[0]!=-10000)
      drawCube(buffer,cubeVerticesImg,verticesPos,cubeFaces,colors);
  }
}
