#include "buffer.h"
#include "draw.h"
#include "3D.h"
#include "drawChunk.h"
#include "actions.h"




//LOADS THE CHUNK TO DISPLAY
void loadChunk(uint8_t *world,uint8_t *chunk,pos playerPos)
{  
  int x,y,z,i,j,k;
  int xmin,ymin,zmin;
  int xmax,ymax,zmax;
  int index=0;
  
  x=(int)floor(playerPos.x/2.0);
  y=(int)floor(playerPos.y/2.0);
  z=(int)floor(playerPos.z/2.0)-1;
  
  xmin=(x<CUBE_CHUNK_X/2) ? 0 : x-CUBE_CHUNK_X/2;
  ymin=(y<CUBE_CHUNK_Y/2) ? 0 : y-CUBE_CHUNK_Y/2;
  zmin=(z<CUBE_CHUNK_Z/2) ? 0 : z-CUBE_CHUNK_Z/2;
  
  xmax=(x+CUBE_CHUNK_X/2>=CUBE_WORLD_X) ? CUBE_WORLD_X-1 : x+CUBE_CHUNK_X/2;
  ymax=(y+CUBE_CHUNK_Y/2>=CUBE_WORLD_Y) ? CUBE_WORLD_Y-1 : y+CUBE_CHUNK_Y/2;
  zmax=(z+CUBE_CHUNK_Z/2>=CUBE_WORLD_Z) ? CUBE_WORLD_Z-1 : z+CUBE_CHUNK_Z/2;
  
  memset(chunk,0,CUBE_CHUNK_X*CUBE_CHUNK_Y*CUBE_CHUNK_Z*sizeof(uint8_t));
  
  for(i=zmin-z+CUBE_CHUNK_Z/2;i<CUBE_CHUNK_Z/2+zmax-z;i++)
  {
    for(j=xmin-x+CUBE_CHUNK_X/2;j<CUBE_CHUNK_X/2+xmax-x;j++)
    {
      for(k=ymin-y+CUBE_CHUNK_Y/2;k<CUBE_CHUNK_Y/2+ymax-y;k++) /*TO DO:use memcpy*/
      {
        index=(i-CUBE_CHUNK_Z/2+z)*CUBE_WORLD_Y*CUBE_WORLD_X+(j-CUBE_CHUNK_X/2+x)*CUBE_WORLD_Y+k-CUBE_CHUNK_Y/2+y;
        chunk[i*CUBE_CHUNK_Y*CUBE_CHUNK_X+j*CUBE_CHUNK_Y+k]=(index%2==0) ? (world[index]) : (256-world[index]);
      }
    }
  }
}


//RANDOM WORLD GENERATOR
void initWorld(uint8_t *world)
{
  int i=0,j=0,rnd=0,height=0;
  memset(world,0,CUBE_WORLD_X*CUBE_WORLD_Y*CUBE_WORLD_Z*sizeof(uint8_t));
  memset(world,1,CUBE_WORLD_X*CUBE_WORLD_Y*sizeof(uint8_t)); //floor 0 (bedrock)
  memset(world+CUBE_WORLD_X*CUBE_WORLD_Y*19,3,CUBE_WORLD_X*CUBE_WORLD_Y*sizeof(uint8_t)); //floor 19 (dirt)
  for(i=0;i<CUBE_WORLD_X*CUBE_WORLD_Y;i++)
  {
    //floor 1 (diamond)
    rnd=random(0,100);
    world[i+CUBE_WORLD_X*CUBE_WORLD_Y]=(rnd<5) ? 6 : 2;
    rnd=random(0,100);
    world[i+CUBE_WORLD_X*CUBE_WORLD_Y*2]=(rnd<3) ? 6 : 2;
    //floor 3 to 6 (gold)
    for(j=0;j<4;j++)
    {
      rnd=random(0,100);
      world[i+CUBE_WORLD_X*CUBE_WORLD_Y*(j+3)]=(rnd<5-j) ? 8 : 2;
    }
    //floor 7 to 12 (iron)
    for(j=7;j<=12;j++)
    {
      rnd=random(0,100);
      world[i+CUBE_WORLD_X*CUBE_WORLD_Y*j]=(rnd<5) ? 9 : 2;
    }
    //floor 13 to 18 (coal)
    for(j=13;j<=18;j++)
    {
      rnd=random(0,100);
      world[i+CUBE_WORLD_X*CUBE_WORLD_Y*j]=(rnd<5) ? 7 : 2;
    }
    //terrain
    height=floor(sin(i%CUBE_WORLD_X*30)*0.49+cos(i/CUBE_WORLD_Y*30)*0.49+sin(i%CUBE_WORLD_X*10+80)*0.49+cos(i/CUBE_WORLD_Y*10+65)*0.49+3);
    for(j=20;j<=19+height-1;j++)
    {
      world[i+CUBE_WORLD_X*CUBE_WORLD_Y*j]=3;
    }
    world[i+CUBE_WORLD_X*CUBE_WORLD_Y*(19+height)]=4;
  }//TO DO:desert, sea, moutain
}


//TRIES TO REACH A NEW POSITION
//and updates if success
void goToPos(uint8_t *chunk,double xPos,double yPos,pos *playerPos)
{
  double newXPos,newYPos;
  newXPos=xPos+playerPos->x-floor(playerPos->x/2)*2;
  newYPos=yPos+playerPos->y-floor(playerPos->y/2)*2;
  if(getCube(chunk,newXPos,newYPos,4)==0)
  {
    if(getCube(chunk,newXPos,newYPos,2)==0)
    {
      playerPos->x+=xPos;
      playerPos->y+=yPos;
    }
    else if(getCube(chunk,newXPos,newYPos,6)==0)
    {
      playerPos->x+=xPos;
      playerPos->y+=yPos;
      playerPos->z+=2;
    }
  }
}

void correctPosition(uint8_t *chunk,pos *playerPos)
{
  if(getCube(chunk,0,0,0)!=0 && ((playerPos->z+0.5)-floor((playerPos->z+0.5)/2)*2)>0.2)
   {playerPos->z=2+floor(playerPos->z/2)*2;}
}



////////////////////////
///////   MAIN   ///////
////////////////////////
///(yes, I need this)///


int main(void)
{
  //CX detection
  if(has_colors)
    lcd_incolor();
  
  //timers
  /*uint16_t time=0,delay=0;
  uint32_t timerAddr=0x900D000C;
  if(has_colors)
    timerAddr=0x900D0004;*/
  
  //bools
  char exit=0,firstDisplay=0;
  char falling=1,repeatTab=0,repeatCtrl=0;
  
  //utils
  void *buffer=malloc(SCREEN_BYTES_SIZE);
  uint8_t *world=malloc(CUBE_WORLD_X*CUBE_WORLD_Y*CUBE_WORLD_Z*sizeof(uint8_t)); //The cube world. Huge array
  uint8_t *chunk=malloc(CUBE_CHUNK_X*CUBE_CHUNK_Y*CUBE_CHUNK_Z*sizeof(uint8_t)); //cube world chunk (kind of a 3D array)
  uint8_t *cubeList=malloc(CUBE_CHUNK_X*CUBE_CHUNK_Y*CUBE_CHUNK_Z*sizeof(uint8_t)); //type of all displayed cubes
  double *cubePos=malloc(CUBE_CHUNK_X*CUBE_CHUNK_Y*CUBE_CHUNK_Z*3*sizeof(double)); //pos of the cubes to display
  int size=0; //number of displayed cubes
  
  //position & camera rotation
  pos playerPos;
  playerPos.x=101.0;
  playerPos.y=101.0;
  playerPos.z=50.1;
  double angleX=5.0,angleZ=10.0;
  
  //inventory ?
  int selectedCube=3;
  
  //initialization
  startRandom();
  initWorld(world);
  
  bufClear(buffer);
  bufDisplay(buffer);
  
  //render loop
  while(!exit)
  {
    //wait for key pressed
    while(!any_key_pressed() && firstDisplay==1 && falling==0)
    {
      idle();
      repeatTab=0;
      repeatCtrl=0;
    }

    //timers update
    //time=*(volatile uint16_t*)timerAddr;
    firstDisplay=1;
    
    //key detection
    if(isKeyPressed(KEY_NSPIRE_ESC))
      exit=1;
      
    if(isKeyPressed(KEY_NSPIRE_UP) && angleX>-80.0)
      angleX-=5.0;
    if(isKeyPressed(KEY_NSPIRE_DOWN) && angleX<80.0)
      angleX+=5.0;
    if(isKeyPressed(KEY_NSPIRE_RIGHT))
      angleZ+=5.0;
    if(isKeyPressed(KEY_NSPIRE_LEFT))
      angleZ-=5.0;
      
    if(isKeyPressed(KEY_NSPIRE_TAB) && repeatTab==0)
    {
      removeBlock(world,chunk,angleX,angleZ,playerPos);
      repeatTab=1;//avoid destroying several blocks in a row
    }
    if(!isKeyPressed(KEY_NSPIRE_TAB))
      repeatTab=0;
    if(isKeyPressed(KEY_NSPIRE_CTRL) && repeatCtrl==0)
    {
      addBlock(world,chunk,angleX,angleZ,playerPos,selectedCube);
      repeatCtrl=1;//avoid placing too much blocks in a row
    }
    if(!isKeyPressed(KEY_NSPIRE_CTRL))
      repeatCtrl=0;
    
      
    if(isKeyPressed(KEY_NSPIRE_8)) //forward
      goToPos(chunk,cos(angleZ-90.0)/1.5,sin(angleZ+90.0)/1.5,&playerPos);
    if(isKeyPressed(KEY_NSPIRE_2) || isKeyPressed(KEY_NSPIRE_5)) //backward
      goToPos(chunk,-cos(angleZ-90.0)/2,-sin(angleZ+90.0)/2,&playerPos);
    if(isKeyPressed(KEY_NSPIRE_6)) //right straff
      goToPos(chunk,cos(angleZ)/1.5,sin(angleZ+180.0)/1.5,&playerPos);
    if(isKeyPressed(KEY_NSPIRE_4)) //left straff
      goToPos(chunk,cos(angleZ-180.0)/1.5,sin(angleZ)/1.5,&playerPos);
      
    if(isKeyPressed(KEY_NSPIRE_PLUS))
      selectedCube=selectedCube%10+1;
    if(isKeyPressed(KEY_NSPIRE_MINUS))
      selectedCube=(selectedCube+8)%10+1;
    
    //gravity
    if(getCube(chunk,0,0,0)==0)
    {
      playerPos.z-=0.5;
      falling=1;
    }
    else
    {
      falling=0;
    }
    correctPosition(chunk,&playerPos);
    
      
    //draw
    loadChunk(world,chunk,playerPos);
    bufClear(buffer);
    
    computeCubeWorldChunk(chunk,cubePos,cubeList,&size,angleX,angleZ,playerPos);
    drawCubeList(buffer,cubePos,cubeList,size,angleX,angleZ);
    
    drawCursor(buffer);
    drawCubeIndicator(buffer,getCubeColor(selectedCube));
    bufDisplay(buffer);
   
    //delay if too fast (!) problem with timers
    /*delay=*(volatile uint16_t*)timerAddr-time;
    while(delay<3000)
    {
      idle();
      delay=*(volatile uint16_t*)timerAddr-time;
    }*/
  }
  
  free(buffer);
  free(cubeList);
  free(world);
  free(chunk);
  free(cubePos);
  return 0;
}
